'use strict';

var _           = require('underscore');
var https = require('https');

var RazahErrors = require('./errors');
var EndpointFactory = require('./helpers/endpoint_factory');

var VALID_DOMAIN = /^(https):\/\/([-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,6}\b)$/;
var HTTP_DOMAIN = /^(http):\/\/([-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,6}\b)$/;

var AUTH_ROUTES = [ /\/v2\/account/,
  /\/v2\/characters/,
  /\/v2\/commerce\/transactions/,
  /\/v2\/pvp\/stats/,
  /\/v2\/pvp\/games/,
  /\/v2\/pvp\/standings/,
  /\/v2\/tokeninfo/,
  /\/v2\/guild\/.*\/log/,
  /\/v2\/guild\/.*\/members/,
  /\/v2\/guild\/.*\/ranks/,
  /\/v2\/guild\/.*\/stash/,
  /\/v2\/guild\/.*\/treasury/,
  /\/v2\/guild\/.*\/teams/,
  /\/v2\/guild\/.*\/upgrades/];

var razahDefaults = {
  domain: 'https://api.guildwars2.com',
  access_token: '',
  lang: 'en'
};

var RazahClient = function(opts) {
  var options = _.extend(razahDefaults, opts);

  this.build = new EndpointFactory(this);

  this.getConfig = function() { return options; };

  this.domain = function() {
    // The domain given must be a valid HTTPS URL
    if(!options.domain.match(VALID_DOMAIN)) {
      var message = "";
      if(options.domain.match(HTTP_DOMAIN)) {
        message = 'API domain must be an HTTPS URL';
      } else {
        message = 'API domain given is invalid';
      }

      throw new RazahErrors.RazahConfigurationInvalid(message);
    }

    return options.domain;
  };

  var needsAuthForUri = function(uri) {
    var auth_required = false;
    _.each(AUTH_ROUTES, function(route) {
      if(route.test(uri)) {
        auth_required = true;
      }
    });

    return auth_required;
  };

  this.request = function(uri, params, callback) {
    if(arguments.length === 1 || (arguments.length === 2 && typeof(params) !== 'function')) {
      callback = _.noop;
      if(typeof params === 'undefined') {
        params = {};
      }
    } else if(arguments.length === 2 && typeof(params) === 'function') {
      callback = params;
      params = {};
    } else if(arguments.length !== 3) {
      throw new RazahErrors.RazahInvalidRequest('Invalid parameters passed to request');
    }

    var response = '';
    var uri_params = '';

    var needsAuth = needsAuthForUri(uri);

    if(needsAuth) {
      params.access_token = options.access_token;
    }

    if(options.lang && options.lang !== 'en') {
      params.lang = options.lang;
    }

    if(Object.keys(params) !== 0) {
      var paramList = [];
      _.each(params, function(val, key) {
        if(typeof(val) === Array) {
          paramList.push(key.toString() + '=' + val.join(','));
        } else {
          paramList.push(key.toString() + '=' + val.toString());
        }
      });

      uri_params = '?' + paramList.join('&');
    }

    var https_url = this.domain() + uri + uri_params;

    console.log(https_url);

    https.get(https_url,
      function(res) {

        var data_response = '';

        res.on('data', function(data) {
          data_response += data;
        });

        res.on('end', function() {
          response = JSON.parse(data_response);

          callback(response);
        });
      }).on('error', function (e) {
        switch(e.statusCode) {
          case 403:
            throw new RazahErrors.RazahAuthenticationFailure("Authentication Failed. The API endpoint '" + uri + "' requires a valid API key");
          case 404:
            RazahErrors.RazahPageNotFound("The page '" + uri + "' was not found on the domain " + this.domain());
          default:
            throw new RazahErrors.RazahRequestError('Exit with status code: ' + e.statusCode + '. Threw error with message ' + e.message);
        }
        console.error(e);
      });
  };

  // Import all v2 endpoints
  this.v2 = (function(client) {
    var APIv2 = require('./v2');
    return new APIv2(client);
  })(this);

  // Expose shortened v2 endpoints for convenience
  this.achievements = this.v2.achievements;
  this.account = this.v2.account;
  this.commerce = this.v2.commerce;
  this.pvp = this.v2.pvp;
  this.guild = this.v2.guild;
  this.game = this.v2.game;
  this.items = this.v2.items;
  this.map = this.v2.maps;
  this.wvw = this.v2.wvw;
  this.other = this.v2.other;

};

// Export Razah to global scope.
if (typeof window !== 'undefined') {
  window.RazahClient = RazahClient;
}

module.exports = RazahClient;

